<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:r="http://software.in2p3.fr/lavoisier/tables.xsd"
                exclude-result-prefixes="xalan">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>

    <xsl:variable name="js-mode" select="'normal'" />

    <xsl:variable name="parameters" select="document('options')/options"/>

    <xsl:variable name="stacking" select="$parameters/@stacking"/>
    <xsl:variable name="axis" select="$parameters/@axis"/>
    <xsl:variable name="unit" select="$parameters/unit/entry"/>
    <xsl:variable name="stacked" select="$parameters/stacked/entry"/>
    <xsl:variable name="types" select="$parameters/types/entry"/>
    <xsl:variable name="type">
        <xsl:choose>
            <xsl:when test="$types[@key='*']/text()"><xsl:value-of select="$types[@key='*']/text()" /></xsl:when>
            <xsl:otherwise><xsl:value-of select="'line'" /></xsl:otherwise>
        </xsl:choose>
    </xsl:variable>

    <xsl:variable name="options" select="$parameters/sections/*"/>

    <xsl:variable name="xAxis" select="$parameters/xAxis" />
    <xsl:variable name="yAxis" select="$parameters/yAxis|$parameters/yAxis2|$parameters/yAxis3" />

    <xsl:variable name="xAxisMode">
        <xsl:choose>
            <xsl:when test="$xAxis/column"> <xsl:value-of select="'xAxis'" /> </xsl:when>
            <xsl:otherwise> <xsl:value-of select="'unit'" /> </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    <xsl:variable name="yAxisMode">
        <xsl:choose>
            <xsl:when test="$yAxis[name(.)='yAxis']/column"> <xsl:value-of select="'yAxis'" /> </xsl:when>
            <xsl:otherwise> <xsl:value-of select="'unit'" /> </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>

    <xsl:variable name="highcharts-colors"
                    xmlns:str="http://exslt.org/strings"
                    select="str:split('4572A7,AA4643,89A54E,80699B,3D96AE,DB843D,92A8CD,A47D7C,B5CA92,4572A7', ',')" />


    <!-- prevents unwanted text() nodes to be printed -->
    <xsl:template match="/r:tables">
        <xsl:if test="$parameters and
            (($options[@id='chart']/*[(@name='type') or (@name='inverted') or (@name='polar')]) or
            ($options[@id='plotOption']//*[@name='stacking']))">
            <xsl:message terminate="yes">chart.type ; chart.inverted ; chart.polar ; plotOptions.{type}.stacking are to be declared in the renderer parameters and not in the advanced options.</xsl:message>
        </xsl:if>

        <supermap>
            <xsl:for-each select="./r:table">
                <map>
                    <!-- TODO: Penser à faire les vérification avant de créer les attributs au cas ou un template aurait un retour vide. -->
                    <xsl:apply-templates select="." mode="CHART"/>
                    <xsl:apply-templates select="." mode="AXIS"/>
                    <xsl:apply-templates select="." mode="SERIES"/>
                    <xsl:apply-templates select="." mode="OPTIONS"/>
                    <property name="title" type="object">
                        <property name="text" type="string">
                            <xsl:choose>
                                <xsl:when test="$options[@id='title']/*[@name='text']"><xsl:value-of select="$options[@id='title']/*[@name='text']" /></xsl:when>
                                <xsl:when test="/r:tables/@title"><xsl:value-of select="concat(/r:tables/@title, ' ', ./@name)" /></xsl:when>
                                <xsl:otherwise><xsl:value-of select="'Untitled'" /></xsl:otherwise>
                            </xsl:choose>
                        </property>
                        <xsl:if test="$options[@id='title']">
                            <xsl:apply-templates select="$options[@id='title']/*[not(@name='text')]" />
                        </xsl:if>
                    </property>
                    <xsl:if test="$js-mode='debug'">
                        <property name="js-advanced-debug" type="object">
                            <property name="parameters" type="object"> <xsl:copy-of select="$parameters" /> </property>
                            <property name="stacking" type="string"> <xsl:value-of select="$stacking" /> </property>
                            <property name="axis" type="string"> <xsl:value-of select="$axis" /> </property>
                            <property name="unit" type="string"> <xsl:copy-of select="$unit" /> </property>
                            <property name="stacked" type="string"> <xsl:copy-of select="$stacked" /> </property>
                            <property name="types" type="object"> <xsl:copy-of select="$types" /> </property>
                            <property name="type" type="string"> <xsl:copy-of select="$type" /> </property>
                            <property name="xAxis" type="object"> <xsl:copy-of select="$xAxis" /> </property>
                            <property name="yAxis" type="object"> <xsl:copy-of select="$yAxis" /> </property>
                            <property name="options" type="object"> <xsl:copy-of select="$options" /> </property>
                        </property>
                    </xsl:if>
                </map>
            </xsl:for-each>
        </supermap>
    </xsl:template>

    <xsl:template match="*" mode="CHART">
        <xsl:if test="$options[@id='chart']/*[(@name='inverted') or (@name='polar') or (@name='type')]">
            <xsl:message terminate="yes">Can not declare 'type', 'inverted' or 'polar' attribute of the chart's section configuration in advanced options</xsl:message>
        </xsl:if>

        <property name="chart" type="object">
            <xsl:choose>
                <xsl:when test="$axis">
                    <property name="inverted" type="boolean"><xsl:value-of select="string($axis='inverted')" /></property>
                    <property name="polar" type="boolean"><xsl:value-of select="string($axis='polar')" /></property>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:call-template name="default"> <xsl:with-param name="wanted" select="'chart'" /> </xsl:call-template>
                </xsl:otherwise>
            </xsl:choose>
            <property name="type" type="string"><xsl:value-of select="$type" /></property>
            <xsl:apply-templates select="$options[@id='chart']/*"/>
        </property>
    </xsl:template>

    <xsl:template match="r:table" mode="AXIS">
        <xsl:if test="not($options[@id='xAxis']/*[@id='title']/*[@key='text'] and $options[@id='xAxis']/*[@id='title']/*[@name='align'] and $options[@id='xAxis']/*[@name='tickmarkPlacement']) and
                        ($options[@id='xAxis']/*[@id='title']/*[@key='text'] or $options[@id='xAxis']/*[@id='title']/*[@name='align'] or $options[@id='xAxis']/*[@name='tickmarkPlacement'])">
            <xsl:message terminate="yes">All theses advanced options for xAxis : 'title.text', 'title.align' and 'tickmarkPlacement' must be specified or none of them.</xsl:message>
        </xsl:if>
        <xsl:if test="not($options[@id='yAxis']/*[@id='labels']/*[@id='style']/*[@name='color'] and $options[@id='yAxis']/*[@id='title']/*[@id='style']/*[@name='color'] and $options[@id='yAxis']/*[@id='title']/*[@name='aling']) and
                        ($options[@id='yAxis']/*[@id='labels']/*[@id='style']/*[@name='color'] or $options[@id='yAxis']/*[@id='title']/*[@id='style']/*[@name='color'] or $options[@id='yAxis']/*[@id='title']/*[@name='aling'])">
            <xsl:message terminate="yes">All theses advanced options for yAxis : 'labels.style.color', 'title.style.color' and 'title.align' must be specified or none of them.</xsl:message>
        </xsl:if>

        <property name="xAxis" type="array">
            <object>
                <property name="categories" type="array">
                    <xsl:for-each select="./r:row">
                        <xsl:apply-templates select="$xAxis[$xAxisMode='xAxis']|ancestor-or-self::r:table/r:column_labels[$xAxisMode='unit']" mode="CATEGORIES">
                            <xsl:with-param name="row" select="." />
                        </xsl:apply-templates>
                    </xsl:for-each>
                </property>
                <xsl:choose>
                    <xsl:when test="$options[@id='xAxis']/*[@id='title']/*[@name='text'] and $options[@id='xAxis']/*[@id='title']/*[@name='align'] and $options[@id='xAxis']/*[@name='tickmarkPlacement']">
                        <xsl:apply-templates select="$options[@id='xAxis']" />
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="default">
                            <xsl:with-param name="wanted" select="'xAxis'" />
                            <xsl:with-param name="additional_object"
                                            select="$xAxis/column[$xAxisMode='xAxis']|ancestor-or-self::r:table/r:column_labels/r:column_label[not(@unit)][$xAxisMode='unit']" />
                        </xsl:call-template>
                        <xsl:apply-templates select="$options[@id='xAxis']/*" />
                    </xsl:otherwise>
                </xsl:choose>
            </object>
        </property>
        <property name="yAxis" type="array">
            <xsl:choose>
                <xsl:when test="$yAxisMode='yAxis'"><xsl:apply-templates select="$yAxis" mode="YAXIS" /></xsl:when>
                <xsl:when test="$yAxisMode='unit'"><xsl:apply-templates select="." mode="YAXIS" /></xsl:when>
            </xsl:choose>
        </property>
    </xsl:template>

    <xsl:template match="r:table" mode="SERIES">
        <property name="series" type="array">
            <xsl:choose>
                <xsl:when test="$yAxisMode='yAxis'">
                    <xsl:apply-templates select="./r:column_labels/r:column_label[text()=$yAxis/*/text()]" mode="SOPTION" />
                </xsl:when>
                <xsl:otherwise>
                    <xsl:apply-templates select="./r:column_labels/r:column_label[@unit]" mode="SUNIT" />
                </xsl:otherwise>
            </xsl:choose>
        </property>
    </xsl:template>

    <xsl:template match="*" mode="OPTIONS">
        <xsl:variable name="highcharts-stacking">
            <xsl:choose>
                <xsl:when test="$stacking='none'">null</xsl:when>
                <xsl:when test="$stacking='normal'">normal</xsl:when>
                <xsl:when test="$stacking='percent'">percent</xsl:when>
                <xsl:otherwise>
                    <!--<xsl:message terminate="yes">Unsupported chart stacking: <xsl:value-of select="$stacking"/></xsl:message>-->
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <property name="plotOptions" type="object">
            <property name="{$type}" type="object">
                <property name="stacking">
                    <xsl:if test="not($highcharts-stacking='null')">
                        <xsl:attribute name="type"><xsl:value-of select="'string'" /></xsl:attribute>
                    </xsl:if>
                    <xsl:value-of select="$highcharts-stacking" />
                </property>
                <xsl:choose>
                    <xsl:when test="$options[@id='plotOptions']/*[@id=$type]"> <xsl:apply-templates select="$options[@id='plotOptions']/*[@id=$type]/*[not(@name='stacking')]" /> </xsl:when>
                    <xsl:otherwise><xsl:call-template name="default"> <xsl:with-param name="wanted" select="'plotOptions'" /> </xsl:call-template></xsl:otherwise>
                </xsl:choose>
            </property>
            <xsl:if test="$options[@id='plotOptions']/*[not(@id=$type)]">
                <xsl:apply-templates select="$options[@id='plotOptions']/*[not(@id=$type)]" />
            </xsl:if>
        </property>
        <xsl:apply-templates select="$options[not(@id='chart') and not(@id='plotOptions') and not(@id='series') and not(@id='title')
                                and not(@id='yAxis') and not(@id='yAxis2') and not(@id='yAxis3') and not(@id='xAxis')]" />
    </xsl:template>

    <!-- xAxis.categories -->
    <xsl:template match="/options/xAxis" mode="CATEGORIES">
        <xsl:param name="row" />
        <string>
            <xsl:for-each select="$xAxis/column">
                <xsl:variable name="axisColumnName" select="." />
                <xsl:if test="position()>1">
                    <!-- separator between column --><xsl:value-of select="'_'" />
                </xsl:if>
                <xsl:value-of select="$row/r:column[count($row/ancestor-or-self::r:table/r:column_labels/r:column_label[text()=$axisColumnName/text()])]/text()" />
            </xsl:for-each>
        </string>
    </xsl:template>
    <xsl:template match="r:column_labels" mode="CATEGORIES">
        <xsl:param name="row" />
        <string>
            <xsl:for-each select="./r:column_label[not(@unit)]">
                <xsl:if test="position()>1">
                    <!-- separator between column --><xsl:value-of select="'_'" />
                </xsl:if>
                <xsl:value-of select="$row/r:column[count(current()/preceding-sibling::r:column_label)+1]/text()" />
            </xsl:for-each>
        </string>
    </xsl:template>
    <!-- yAxis-->
    <xsl:template match="/options/yAxis|/options/yAxis2|/options/yAxis3" mode="YAXIS">
        <xsl:variable name="unit" select="@unit"/>
        <xsl:variable name="yAxisName" select="name(.)" />
        <xsl:variable name="color">
            <xsl:choose>
                <xsl:when test="$yAxisName='yAxis'"><xsl:value-of select="$highcharts-colors[1]" /></xsl:when>
                <xsl:when test="$yAxisName='yAxis2'"><xsl:value-of select="$highcharts-colors[2]" /></xsl:when>
                <xsl:when test="$yAxisName='yAxis3'"><xsl:value-of select="$highcharts-colors[3]" /></xsl:when>
                <xsl:otherwise><xsl:value-of select="$highcharts-colors[1]" /></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:if test="./column">
            <object>
                <xsl:if test="not($options[@id=$yAxisName]/*[@name='opposite'])">
                    <xsl:if test="position()>1">
                        <property name="opposite" type="boolean">true</property>
                    </xsl:if>
                </xsl:if>
                <xsl:choose>
                    <xsl:when test="$options[@id=$yAxisName]">
                        <xsl:apply-templates select="$options[@id=$yAxisName]/*[not(@name='opposite')]" />
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="default">
                            <xsl:with-param name="wanted" select="'yAxis'" />
                            <xsl:with-param name="additional_object" select="$unit" />
                            <xsl:with-param name="yAxisColor" select="$color" />
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </object>
        </xsl:if>
    </xsl:template>
    <xsl:template match="r:table" mode="YAXIS">
        <xsl:variable name="highcharts-units" select="ancestor-or-self::r:table/r:column_labels/r:column_label[not(preceding-sibling::r:column_label/@unit = @unit)]/@unit" />

        <xsl:for-each select="$highcharts-units">
            <xsl:variable name="unit" select="@unit"/>
            <xsl:variable name="color" select="$highcharts-colors
[position() = count(./r:column_labels/r:column_label[@unit=$unit][1]/preceding-sibling::r:column_label[not(@unit)])+1]" />

            <object>
                <xsl:if test="position()>1">
                    <property name="opposite" type="boolean">true</property>
                </xsl:if>
                <xsl:call-template name="default">
                    <xsl:with-param name="wanted" select="'yAxis'" />
                    <xsl:with-param name="additional_object" select="$unit" />
                    <xsl:with-param name="yAxisColor" select="$color" />
                </xsl:call-template>
            </object>
        </xsl:for-each>
    </xsl:template>

    <!-- SERIES -->
    <xsl:template match="r:table/r:column_labels/r:column_label" mode="SOPTION">
        <xsl:variable name="text" select="./text()" />
        <xsl:variable name="unitNumber">
            <xsl:for-each select="$yAxis">
                <xsl:if test="./column/text() = $text">
                    <xsl:value-of select="position()"/>
                </xsl:if>
            </xsl:for-each>
        </xsl:variable>

        <xsl:if test="$unitNumber">
            <object>
                <property name="data" type="array">
                    <xsl:for-each select="ancestor-or-self::r:table/r:row/r:column[count(ancestor-or-self::r:column_label[text()=$text])+1]">
                        <number><xsl:value-of select="text()"/></number>
                    </xsl:for-each>
                </property>
                <xsl:call-template name="default">
                    <xsl:with-param name="wanted" select="'series'" />
                    <xsl:with-param name="additional_object" select="$types[@key=$text]/text()" />
                    <xsl:with-param name="seriesUnitNumber" select="$unitNumber" />
                </xsl:call-template>
            </object>
        </xsl:if>
    </xsl:template>
    <xsl:template match="r:table/r:column_labels/r:column_label[@unit]" mode="SUNIT">
        <xsl:variable name="unit" select="@unit" />
        <xsl:variable name="highcharts-units"
                      select="ancestor-or-self::r:table/r:column_labels/r:column_label[@unit][not(preceding-sibling::r:column_label/@unit = @unit)]" />
        <xsl:variable name="unitNumber">
            <xsl:for-each select="$highcharts-units">
                <xsl:if test="@unit = $unit">
                    <xsl:value-of select="position()"/>
                </xsl:if>
            </xsl:for-each>
        </xsl:variable>

        <object>
            <property name="data" type="array">
                <xsl:for-each select="ancestor-or-self::r:table/r:row/r:column[count(current()/preceding-sibling::r:column_label)+1]">
                    <number><xsl:value-of select="text()"/></number>
                </xsl:for-each>
            </property>
            <xsl:call-template name="default">
                <xsl:with-param name="wanted" select="'series'" />
                <xsl:with-param name="seriesUnitNumber" select="$unitNumber" />
                <xsl:with-param name="additional_object" />
            </xsl:call-template>
        </object>
    </xsl:template>

    <!-- DEFAULT VALUES -->
    <xsl:template name="default">
        <xsl:param name="wanted" />
        <xsl:param name="additional_object" select="_" />

        <!-- Specially for yAxis -->
        <xsl:param name="yAxisColor" select="_" />
        <!-- Specially for series -->
        <xsl:param name="seriesUnitNumber" select="_" />

        <xsl:choose>
            <xsl:when test="$wanted='chart'">
                <property name="inverted" type="boolean">false</property>
                <property name="polar" type="boolean">false</property>
            </xsl:when>
            <xsl:when test="$wanted='plotOptions'">
                <property name="pointPadding" type="number">0</property>
                <property name="groupPadding" type="number">0</property>
            </xsl:when>
            <xsl:when test="$wanted='xAxis'">
                <xsl:if test="not($type='column') and $axis='polar'">
                    <property name="tickmarkPlacement" type="string">on</property>
                </xsl:if>
                <property name="title" type="object">
                    <xsl:if test="not($axis='polar')">
                        <property name="text" type="string">
                            <xsl:for-each select="$additional_object">
                                <xsl:if test="position()>1">
                                    <!-- separator between column --> <xsl:value-of select="'_'" />
                                </xsl:if>
                                <xsl:value-of select="text()" />
                            </xsl:for-each>
                        </property>
                    </xsl:if>
                    <property name="align" type="string">high</property>
                </property>
            </xsl:when>
            <xsl:when test="$wanted='yAxis'">
                <property name="labels" type="object">
                    <property name="style" type="object">
                        <property name="color" type="string"><xsl:value-of select="concat('#', $yAxisColor)" /></property>
                    </property>
                </property>
                <property name="title" type="object">
                    <property name="style" type="object">
                        <property name="color" type="string"><xsl:value-of select="concat('#', $yAxisColor)" /></property>
                    </property>
                    <property name="align" type="string">high</property>
                </property>
            </xsl:when>
            <xsl:when test="$wanted='series'">
                <property name="name" type="string"><xsl:value-of select="text()" /></property>
                <xsl:if test="$additional_object">
                    <property name="type" type="string"><xsl:value-of select="$additional_object" /></property>
                </xsl:if>
                <property name="yAxis" type="number"><xsl:value-of select="$seriesUnitNumber - 1" /></property>
            </xsl:when>
        </xsl:choose>
    </xsl:template>

    <!-- Additional 'section' and 'option' -->
    <xsl:template match="section">
        <xsl:choose>
            <xsl:when test="./section[@id='item']">
                <property name="{@id}" type="array">
                    <xsl:for-each select="./section[@id='item']">
                        <object> <xsl:apply-templates select="./*" /> </object>
                    </xsl:for-each>
                </property>
            </xsl:when>
            <xsl:when test="./option[@name='item']">
                <property name="{@id}" type="array"><xsl:apply-templates select="*" /></property>
            </xsl:when>
            <xsl:otherwise> <property name="{@id}" type="object"> <xsl:apply-templates select="*" /> </property> </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="option[not(@name='item')]">
        <property name="{@name}">
            <xsl:choose>
                <xsl:when test="number(text())=text()"><xsl:attribute name="type"><xsl:value-of select="'number'" /></xsl:attribute></xsl:when>
                <xsl:when test="text()='true' or text()='false'"> <xsl:attribute name="type"><xsl:value-of select="'boolean'" /></xsl:attribute></xsl:when>
                <xsl:when test="not(text()='null' or text()='undefined')"> <xsl:attribute name="type"><xsl:value-of select="'string'" /></xsl:attribute></xsl:when>
            </xsl:choose>
            <xsl:value-of select="text()" />
        </property>
    </xsl:template>
    <xsl:template match="option[@name='item']">
        <xsl:choose>
            <xsl:when test="number(text())=text()"> <number> <xsl:value-of select="text()" /> </number> </xsl:when>
            <xsl:when test="text()='true' or text()='false'"> <boolean><xsl:value-of select="text()" /> </boolean> </xsl:when>
            <xsl:when test="not(text()='null' or text()='undefined')"> <string> <xsl:value-of select="text()" /> </string> </xsl:when>
            <xsl:otherwise> <value> <xsl:value-of select="text()" /> </value> </xsl:otherwise>
        </xsl:choose>

    </xsl:template>
</xsl:stylesheet>
